SystemOrganization addCategory: #'Neo-Console-Core'!

NonInteractiveTranscript subclass: #NeoConsoleTranscript
	instanceVariableNames: 'date'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Neo-Console-Core'!

!NeoConsoleTranscript commentStamp: 'SvenVanCaekenberghe 8/25/2015 17:28' prior: 0!
I am NeoConsoleTranscript, a NonInteractiveTranscript that writes to a dated file, switching to a new file each day.

	(NeoConsoleTranscript onFileNamed: 'server-{1}.log') install.
!

!NeoConsoleTranscript class methodsFor: 'constants' stamp: 'SvenVanCaekenberghe 8/25/2015 17:26'!
defaultLogFileName
	"The default file name that will be used to write to.
	This needs to have a slot where the date will come."
	
	^ 'PharoTranscript-{1}.log'! !

!NeoConsoleTranscript methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/25/2015 17:26'!
fileNameWithDate
	| newName |
	newName := self fileName format: { date yyyymmdd }.
	self 
		assert: self fileName ~= newName 
		description: 'The filename should include a {1} slot for the date'.
	^ newName! !

!NeoConsoleTranscript methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 8/25/2015 17:26'!
initialize
	super initialize.
	date := Date today! !

!NeoConsoleTranscript methodsFor: 'private' stamp: 'SvenVanCaekenberghe 8/25/2015 17:26'!
initializeStream
	"Open the file stream that I write to or connect to #stdout.
	I use the proper line end convention.
	I will append to regular files.
	Overwrtitten to use #fileNameWithDate"
	
	stream := self isStdout 
		ifTrue: [ FileStream stdout ]
		ifFalse: [ FileStream fileNamed: self fileNameWithDate ].
	stream wantsLineEndConversion: true.
	self isStdout
		ifTrue: [ stream converter "Force lazy initialization of converter" ] 
		ifFalse: [ stream setToEnd ].
	^ stream! !

!NeoConsoleTranscript methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/25/2015 17:26'!
stream 
	"Overwritten to check if the date changed on each access"
	
	Date today > date
		ifTrue: [ 
			"The date changed: close the old file and advance the date"
			self close.
			date := Date today ].
	"Since stream is now nil, #initializeStream will be called"
	^ super stream! !

Object subclass: #NeoConsole
	instanceVariableNames: ''
	classVariableNames: 'History Results'
	poolDictionaries: ''
	category: 'Neo-Console-Core'!

!NeoConsole commentStamp: 'SvenVanCaekenberghe 6/17/2015 15:25' prior: 0!
I am a class facade offering access to some global state held by the NeoConsole package.

I hold the history of all evaluated commands.

I hold the last 3 results.!

!NeoConsole class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/18/2015 00:03'!
clearResults
	^ self results 
		atAllPut: nil; 
		yourself! !

!NeoConsole class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/17/2015 15:25'!
history
	^ History ifNil: [ History := OrderedCollection new ]! !

!NeoConsole class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/17/2015 15:33'!
last
	^ self results last! !

!NeoConsole class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/17/2015 17:56'!
lastResult: anObject
	| results |
	results := self results.
	anObject = results ifTrue: [ ^ results ].
	results at: 1 put: (results at: 2).
	results at: 2 put: (results at: 3).
	results at: 3 put: anObject.
	^ results! !

!NeoConsole class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/17/2015 15:26'!
results
	^ Results ifNil: [ Results := Array new: 3 ]! !

Object subclass: #NeoConsoleCommand
	instanceVariableNames: 'arguments result'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Neo-Console-Core'!

!NeoConsoleCommand commentStamp: 'SvenVanCaekenberghe 2/24/2015 14:33' prior: 0!
I am NeoConsoleCommand. I describe a command that can be executed by the NeoConsole infrastructure. I am abstract.!

!NeoConsoleCommand class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/24/2015 15:40'!
defaultCommand
	^ NeoConsoleEvalCommand ! !

!NeoConsoleCommand class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/24/2015 15:44'!
forLine: line
	| commandClass |
	commandClass := self knownCommands 
		detect: [ :each | each handlesLine: line ] 
		ifNone: [ self defaultCommand ].
	^ commandClass! !

!NeoConsoleCommand class methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 2/24/2015 15:35'!
handlesLine: line
	^ line beginsWith: self prefix! !

!NeoConsoleCommand class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/24/2015 15:19'!
helpText
	^ self name, String cr, self comment! !

!NeoConsoleCommand class methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 2/24/2015 15:31'!
isMultiLine
	^ self isSingleLine not! !

!NeoConsoleCommand class methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 2/24/2015 15:31'!
isSingleLine
	^ true! !

!NeoConsoleCommand class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/24/2015 15:36'!
knownCommands
	^ self subclasses! !

!NeoConsoleCommand class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/24/2015 15:15'!
prefix
	^ self subclassResponsibility ! !

!NeoConsoleCommand methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/24/2015 16:43'!
argument
	^ arguments first! !

!NeoConsoleCommand methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/24/2015 14:34'!
arguments
	^ arguments ! !

!NeoConsoleCommand methodsFor: 'public' stamp: 'SvenVanCaekenberghe 2/24/2015 15:17'!
execute
	self subclassResponsibility ! !

!NeoConsoleCommand methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 2/24/2015 16:09'!
hasArguments
	^ arguments notNil and: [ arguments notEmpty ]! !

!NeoConsoleCommand methodsFor: 'initialization-release' stamp: 'SvenVanCaekenberghe 2/24/2015 16:29'!
on: input
	arguments := Array with: input trimBoth! !

!NeoConsoleCommand methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/24/2015 15:38'!
result
	^ result! !

NeoConsoleCommand subclass: #NeoConsoleDescribeCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Neo-Console-Core'!

!NeoConsoleDescribeCommand commentStamp: 'SvenVanCaekenberghe 8/25/2015 17:28' prior: 0!
Describe the result of the last evaluation
showing the structure of that object

You can use the single character alias =

describe - describe the result of the last evaluation!

!NeoConsoleDescribeCommand class methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/18/2015 09:15'!
handlesLine: line
	^ line first = $= or: [ super handlesLine: line ]! !

!NeoConsoleDescribeCommand class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/17/2015 22:47'!
prefix
	^ 'describe'! !

!NeoConsoleDescribeCommand methodsFor: 'public' stamp: 'SvenVanCaekenberghe 6/17/2015 23:49'!
execute
	| object |
	object := NeoConsole last.
	result := String streamContents: [ :out |
		out << 'self: '; << (self printLimited: [ object ]); cr.
		out << 'class: '; << object class name; cr.
		object class allInstVarNames do: [ :each |
			out << each; << ': '; << (self printLimited: [ object instVarNamed: each ]); cr ].
		object class isVariable ifTrue: [
			object size <= 20
				ifTrue: [
					1 to: object size do: [ :index |
						out print: index; << ': '; << (self printLimited: [ object at: index ]); cr ] ]
				ifFalse: [ 
					1 to: 10 do: [ :index |
						out print: index; << ': '; << (self printLimited: [ object at: index ]); cr ].
					out << '...'; cr.
					object size - 10 to: object size do: [ :index |
						out print: index; << ': '; << (self printLimited: [ object at: index ]); cr ] ] ] ]! !

!NeoConsoleDescribeCommand methodsFor: 'initialization-release' stamp: 'SvenVanCaekenberghe 6/17/2015 20:57'!
on: input
	super on: input.
	arguments := self argument findTokens: Character separators.
	arguments := arguments allButFirst ! !

!NeoConsoleDescribeCommand methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/17/2015 23:47'!
printLimited: valuable 
	^ [ valuable value printStringLimitedTo: 64 ] on: Error do: [ '<error while printing>' ]! !

NeoConsoleCommand subclass: #NeoConsoleEvalCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Neo-Console-Core'!

!NeoConsoleEvalCommand commentStamp: 'SvenVanCaekenberghe 6/17/2015 15:42' prior: 0!
Evaluate a Pharo expression

eval <expression>
<expression>*
<empty line>
- evaluate a multi line expression and show the result

Eval is normally the default command, no need to type 'eval'!

!NeoConsoleEvalCommand class methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 2/24/2015 15:31'!
isSingleLine
	^ false! !

!NeoConsoleEvalCommand class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/24/2015 15:18'!
prefix
	^ 'eval'! !

!NeoConsoleEvalCommand methodsFor: 'public' stamp: 'SvenVanCaekenberghe 10/29/2015 14:48'!
execute
	result := String 
		streamContents: [ :out | 
			[ 	| compiler evaluationResult |
				compiler := self class compiler.
				NeoConsole history add: self argument.
				evaluationResult := compiler evaluate: self argument.
				NeoConsole lastResult: evaluationResult. 
				out print: evaluationResult ]
					on: Error
					do: [ :exception |
						out print: exception; cr.
						exception signalerContext printDetails: out.
						exception signalerContext sender debugStack: 8 on: out ] ]
		limitedTo: self outputLimit.
	result size < self outputLimit 
		ifFalse: [ result := result , ' ... (truncated) ...' ]! !

!NeoConsoleEvalCommand methodsFor: 'initialization-release' stamp: 'SvenVanCaekenberghe 2/24/2015 16:50'!
on: input
	super on: input.
	(self argument beginsWith: self class prefix)
		ifTrue: [ arguments := Array with: (self argument allButFirst: self class prefix size) trimBoth ]
	! !

!NeoConsoleEvalCommand methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/16/2015 16:26'!
outputLimit
	^ 16384! !

NeoConsoleCommand subclass: #NeoConsoleGetCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Neo-Console-Core'!

!NeoConsoleGetCommand commentStamp: 'SvenVanCaekenberghe 6/17/2015 15:42' prior: 0!
Get the value of a metric (gauge or counter)

get - list all metrics
get <metric> - show the current value of <metric>!

!NeoConsoleGetCommand class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/24/2015 15:18'!
prefix
	^ 'get'! !

!NeoConsoleGetCommand methodsFor: 'public' stamp: 'SvenVanCaekenberghe 8/17/2015 12:01'!
execute
	self hasArguments 
		ifTrue: [ 
			[ result := (NeoConsoleMetric named: self argument) value asString ] 
				on: NotFound 
				do: [ :exception | result := exception printString ] ]
		ifFalse: [ 
			result := String streamContents: [ :out |
				out << 'known metrics:'; cr.
				NeoConsoleMetric knownMetrics 
					do: [ :each | out space; space; << each name; << ' - '; << each description ] 
					separatedBy: [ out cr ] ] ]! !

!NeoConsoleGetCommand methodsFor: 'initialization-release' stamp: 'SvenVanCaekenberghe 2/24/2015 17:30'!
on: input
	super on: input.
	arguments := self argument findTokens: Character separators.
	arguments := arguments allButFirst ! !

NeoConsoleCommand subclass: #NeoConsoleHelpCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Neo-Console-Core'!

!NeoConsoleHelpCommand commentStamp: 'SvenVanCaekenberghe 6/17/2015 15:42' prior: 0!
Get some minimal help

help - list all commands
help <command> - print help about <command>!

!NeoConsoleHelpCommand class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/24/2015 15:18'!
prefix
	^ 'help'! !

!NeoConsoleHelpCommand methodsFor: 'public' stamp: 'SvenVanCaekenberghe 6/16/2015 16:16'!
execute
	self hasArguments 
		ifTrue: [ 
			(NeoConsoleCommand knownCommands 
					detect: [ :each | each prefix = self argument ] ifNone: [ nil ])
				ifNil: [ result := 'unknown command <{1}>' format: { self argument } ] 
				ifNotNil: [ :command | result := command helpText ] ]
		ifFalse: [ 
			result := String streamContents: [ :out |
				out << 'help <command>'; cr.
				out << 'known commands are:'; cr.
				NeoConsoleCommand knownCommands 
					do: [ :each | 
						out space; space; << each prefix.
						NeoConsoleCommand defaultCommand = each ifTrue: [ out << ' DEFAULT' ] ] 
					separatedBy: [ out cr ] ] ]! !

!NeoConsoleHelpCommand methodsFor: 'initialization-release' stamp: 'SvenVanCaekenberghe 2/24/2015 16:48'!
on: input
	super on: input.
	arguments := self argument findTokens: Character separators.
	arguments := arguments allButFirst ! !

NeoConsoleCommand subclass: #NeoConsoleHistoryCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Neo-Console-Core'!

!NeoConsoleHistoryCommand commentStamp: 'SvenVanCaekenberghe 6/18/2015 13:05' prior: 0!
History of evaluation input previously entered

You can use the single character alias !! even without a space

history - print a summary of the history
history all - print the full history
history clear - clear the full history
history <integer> - execute a previous input again
history last - execute the last input again
history !! - execute the last input again!

!NeoConsoleHistoryCommand class methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/18/2015 09:14'!
handlesLine: line
	^ line first = $!! or: [ super handlesLine: line ]! !

!NeoConsoleHistoryCommand class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/17/2015 15:41'!
prefix
	^ 'history'! !

!NeoConsoleHistoryCommand methodsFor: 'public' stamp: 'SvenVanCaekenberghe 6/18/2015 09:53'!
execute
	self hasArguments
		ifTrue: [
			self argument = 'all' 
				ifTrue: [ self history ].
			self argument = 'clear' 
				ifTrue: [ self historyClear ].
			(Integer readFrom: self argument ifFail: [ nil ]) 
				ifNotNil: [ :index | self executeHistoryAt: index ].
			(#(last !!) includes: self argument)
				ifTrue: [ self executeHistoryLast ] ]
		ifFalse: [ self historySummary ]! !

!NeoConsoleHistoryCommand methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/17/2015 17:05'!
executeHistoryAt: index
	| input evalCommand |
	[ 
	input := NeoConsole history at: index.
	evalCommand := NeoConsoleEvalCommand new on: input.
	evalCommand execute.
	result := input , String cr , String cr , evalCommand result ]
		on: SubscriptOutOfBounds
		do: [ result := 'no history item @ ' , index asString ]! !

!NeoConsoleHistoryCommand methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/18/2015 09:54'!
executeHistoryLast
	| size |
	size := NeoConsole history size.
	size isZero 
		ifTrue: [ result := 'history is empty' ]
		ifFalse: [ self executeHistoryAt: size ]! !

!NeoConsoleHistoryCommand methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/18/2015 09:19'!
history
	result := String streamContents: [ :out |
		1 to: NeoConsole history size do: [ :each |
			out print: each; << ': '; << (NeoConsole history at: each); cr ] ]! !

!NeoConsoleHistoryCommand methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/17/2015 17:06'!
historyClear
	NeoConsole history removeAll.
	result := 'history cleared'! !

!NeoConsoleHistoryCommand methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/18/2015 09:19'!
historySummary
	result := String streamContents: [ :out | | size |
		size := NeoConsole history size.
		(1 max: (size - self historySummarySize)) to: size do: [ :each |
			out print: each; << ': '; << (NeoConsole history at: each); cr ] ]! !

!NeoConsoleHistoryCommand methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/17/2015 17:08'!
historySummarySize
	^ 10! !

!NeoConsoleHistoryCommand methodsFor: 'initialization-release' stamp: 'SvenVanCaekenberghe 6/18/2015 13:01'!
on: input
	super on: input.
	self argument first = $!!
		ifTrue: [ 
			arguments := self argument allButFirst findTokens: Character separators ]
		ifFalse: [ 
			arguments := self argument findTokens: Character separators.
			arguments := arguments allButFirst ]! !

NeoConsoleCommand subclass: #NeoConsoleQuitCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Neo-Console-Core'!

!NeoConsoleQuitCommand commentStamp: 'SvenVanCaekenberghe 6/17/2015 15:42' prior: 0!
Quit

quit - Quit the REPL
!

!NeoConsoleQuitCommand class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/24/2015 15:36'!
prefix
	^ 'quit'! !

!NeoConsoleQuitCommand methodsFor: 'public' stamp: 'SvenVanCaekenberghe 2/24/2015 15:38'!
execute
	result := 'Bye!!'.
	^ nil! !

Object subclass: #NeoConsoleEnvironment
	instanceVariableNames: 'environment'
	classVariableNames: 'UniqueInstance'
	poolDictionaries: ''
	category: 'Neo-Console-Core'!

!NeoConsoleEnvironment commentStamp: 'SvenVanCaekenberghe 9/7/2015 16:52' prior: 0!
NeoConsoleEnvironment holds a number of configuration key/value pairs and can be used for system configuration during startup. NeoConsoleEnvironment offers a single, central place for configuration.

For example, in your run/startup script you could write: 

NeoConsoleEnvironment current
	at: #DatabaseUsername put: 'webapp';
	at: #DatabasePassword put: 'secret'.
	
And use those values when you make a database connection once your application starts up.

You can optionally request to load the key/value of your OS environment using #loadOSEnvironment.

NeoConsoleEnvironment current
	loadOSEnvironment;
	yourself.

Keys are stored as symbols.
!

!NeoConsoleEnvironment class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/7/2015 14:45'!
current
	^ UniqueInstance ifNil: [ UniqueInstance := self new ]! !

!NeoConsoleEnvironment class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/7/2015 15:20'!
default
	^ self current! !

!NeoConsoleEnvironment methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/7/2015 15:15'!
at: key
	"Return my value for key.
	Signal NotFound if I do not know key."
	
	^ environment at: key! !

!NeoConsoleEnvironment methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/7/2015 15:15'!
at: key ifAbsent: block
	"Return my value for key.
	Return the value of evaluating block in case I do not know key."
	
	^ environment at: key ifAbsent: block! !

!NeoConsoleEnvironment methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/7/2015 15:16'!
at: key ifAbsentPut: block
	"Return my value for key.
	Store and return the value of evaluating block in case I do not know key."

	^ environment at: key ifAbsentPut: block! !

!NeoConsoleEnvironment methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/7/2015 15:17'!
at: key ifPresent: block
	"Return the value of evaluating block with my value for key.
	Return self, in effect do nothing, in case I do not know key."

	^ environment at: key ifPresent: block! !

!NeoConsoleEnvironment methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/7/2015 15:20'!
at: key put: value
	"I will store value for key.
	Return value."
	
	^ environment at: key asSymbol put: value! !

!NeoConsoleEnvironment methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/7/2015 16:50'!
includesKey: key
	^ environment includesKey: key! !

!NeoConsoleEnvironment methodsFor: 'initialization' stamp: 'SvenVanCaekenberghe 9/7/2015 14:46'!
initialize
	environment := Dictionary new! !

!NeoConsoleEnvironment methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/7/2015 16:48'!
keys
	^ environment keys! !

!NeoConsoleEnvironment methodsFor: 'enumeration' stamp: 'SvenVanCaekenberghe 9/7/2015 16:49'!
keysAndValuesDo: block
	environment keysAndValuesDo: block! !

!NeoConsoleEnvironment methodsFor: 'initialization' stamp: 'SvenVanCaekenberghe 9/7/2015 16:50'!
loadOSEnvironment
	OSPlatform current environment 
		keysAndValuesDo: [ :key :value | 
			self at: key put: value ]! !

Object subclass: #NeoConsoleMetric
	instanceVariableNames: 'name description reader'
	classVariableNames: 'KnownMetrics'
	poolDictionaries: ''
	category: 'Neo-Console-Core'!

!NeoConsoleMetric commentStamp: 'SvenVanCaekenberghe 2/24/2015 17:16' prior: 0!
I am NeoConsoleMetric. I represent a named metric with a description. I evaluate my reader to access my actual value.

My class side holds a list of #knownMetrics.!

!NeoConsoleMetric class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/16/2015 19:20'!
add: metric
	^ self knownMetrics 
		removeAllSuchThat: [ :each | each name = metric name ]; 
		add: metric! !

!NeoConsoleMetric class methodsFor: 'initialization' stamp: 'SvenVanCaekenberghe 8/17/2015 13:42'!
initialize
	"self initialize"
	
	KnownMetrics := nil.
	self initializeMetrics.
	
	"last update: 2015-08-17"! !

!NeoConsoleMetric class methodsFor: 'initialization' stamp: 'SvenVanCaekenberghe 6/16/2015 17:06'!
initializeChronologyMetrics
	self add: (NeoConsoleMetric 
		named: 'system.date' 
		description: 'Current date' 
		reader: [ Date today printString ]).
	self add: (NeoConsoleMetric
		named: 'system.time'
		description: 'Current time'
		reader: [ Time now printString ]).
	self add: (NeoConsoleMetric
		named: 'system.timestamp'
		description: 'Current timestamp'
		reader: [ DateAndTime now printString ]).! !

!NeoConsoleMetric class methodsFor: 'initialization' stamp: 'SvenVanCaekenberghe 6/16/2015 17:03'!
initializeMemoryMetrics
	self add: (NeoConsoleMetric 
		named: 'memory.total' 
		description: 'Total allocated memory' 
		reader: [ Smalltalk vm memoryEnd ]).
	self add: (NeoConsoleMetric
		named: 'memory.free'
		description: 'Free memory'
		reader: [ Smalltalk vm memoryEnd - Smalltalk vm youngSpaceEnd ])! !

!NeoConsoleMetric class methodsFor: 'initialization' stamp: 'SvenVanCaekenberghe 8/17/2015 11:46'!
initializeMetrics
	self initializeSystemStatusMetric.
	self initializeMemoryMetrics.
	self initializeChronologyMetrics.
	self initializeProcessMetrics.
	self initializeVersionMetrics! !

!NeoConsoleMetric class methodsFor: 'initialization' stamp: 'SvenVanCaekenberghe 8/17/2015 13:37'!
initializeProcessMetrics
	self add: (NeoConsoleMetric 
		named: 'process.count' 
		description: 'Current process count' 
		reader: [ (Process allSubInstances reject: #isTerminated) size asString ]).
	self add: (NeoConsoleMetric
		named: 'process.list'
		description: 'Current list of processes'
		reader: [ String cr join: ((Process allSubInstances reject: #isTerminated) collect: #name as: Array) ])! !

!NeoConsoleMetric class methodsFor: 'initialization' stamp: 'SvenVanCaekenberghe 8/14/2015 16:43'!
initializeSystemStatusMetric
	self add: (NeoConsoleMetric 
		named: 'system.status' 
		description: 'Simple system status' 
		reader: [ | vm |
			vm :=SmalltalkImage current vm. 
			'Status OK - Clock {1} - Allocated {2} bytes - {3} % free.' format: { 
				DateAndTime now.
				vm memoryEnd asStringWithCommas. 
				(vm memoryEnd - vm youngSpaceEnd / vm memoryEnd * 100.0) printShowingDecimalPlaces: 2 } ])! !

!NeoConsoleMetric class methodsFor: 'initialization' stamp: 'SvenVanCaekenberghe 8/17/2015 11:46'!
initializeVersionMetrics
	self add: (NeoConsoleMetric 
		named: 'system.version' 
		description: 'Image version info' 
		reader: [ SystemVersion current ]).
	self add: (NeoConsoleMetric 
		named: 'system.mcversions' 
		description: 'Monticello packages version info' 
		reader: [ String cr join: (MCWorkingCopy allManagers collect: #description) sorted ])! !

!NeoConsoleMetric class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/24/2015 17:15'!
knownMetrics
	^ KnownMetrics ifNil: [ KnownMetrics := OrderedCollection new ]! !

!NeoConsoleMetric class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/16/2015 18:22'!
named: metricName
	^ self knownMetrics detect: [ :each | each name = metricName ] ifNone: [ NotFound signalFor: metricName ]! !

!NeoConsoleMetric class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 2/24/2015 17:18'!
named: metricName description: description reader: valueable
	^ self new
		name: metricName;
		description: description;
		reader: valueable;
		yourself! !

!NeoConsoleMetric methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/24/2015 17:13'!
description
	^ description! !

!NeoConsoleMetric methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/24/2015 17:14'!
description: string
	description := string! !

!NeoConsoleMetric methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/24/2015 17:13'!
name
	^ name! !

!NeoConsoleMetric methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/24/2015 17:14'!
name: string
	name := string! !

!NeoConsoleMetric methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 6/16/2015 18:23'!
printOn: stream
	super printOn: stream.
	stream nextPut: $(; << name; nextPut: $)! !

!NeoConsoleMetric methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/24/2015 17:13'!
reader
	^ reader! !

!NeoConsoleMetric methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/24/2015 17:14'!
reader: aValuable
	reader := aValuable! !

!NeoConsoleMetric methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/24/2015 17:14'!
value
	^ reader value! !

Object subclass: #NeoConsoleMetricDelegate
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Neo-Console-Core'!

!NeoConsoleMetricDelegate commentStamp: 'SvenVanCaekenberghe 8/14/2015 17:14' prior: 0!
NeoConsoleMetricDelegate is a Zinc HTTP Components Server delegate that offers HTTP web service access to all metrics.

Start a safe (locally bound) & dedicated HTTP server using: 

	NeoConsoleMetricDelegate startOn: 1707.

Try GET http://localhost:1707/metrics/system.status 

	ZnClient new get: 'http://localhost:1707/metrics/system.status'
	
To list all metrics GET /metrics
!

!NeoConsoleMetricDelegate class methodsFor: 'public' stamp: 'SvenVanCaekenberghe 8/26/2015 00:43'!
startOn: port
	"self startOn: 1707"
	
	^ (ZnServer on: port)
		delegate: self new;
		bindingAddress: NetNameResolver loopBackAddress;
		logLevel: 2;
		start;
		yourself! !

!NeoConsoleMetricDelegate methodsFor: 'public' stamp: 'SvenVanCaekenberghe 8/14/2015 22:04'!
handleRequest: request
	(request method = #GET and: [ request uri firstPathSegment = #metrics ])
		ifFalse: [ ^ ZnResponse notFound: request uri ].
	request uri pathSegments size = 1
		ifTrue: [ ^ self responseForKnowMetrics ].
	^ [ self responseForMetricNamed: request uri lastPathSegment ]
		on: NotFound
		do: [ ZnResponse notFound: request uri ]! !

!NeoConsoleMetricDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 8/17/2015 13:36'!
okResponse: text
	| output |
	output := String new: text size + 10 streamContents: [ :out |
		text lines do: [ :each | out << each ] separatedBy: [ out lf ].
		out lf ].
	^ ZnResponse ok: (ZnEntity text: output)! !

!NeoConsoleMetricDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 8/17/2015 13:37'!
responseForKnowMetrics
	| text |
	text := String cr join: (NeoConsoleMetric knownMetrics collect: [ :each |
		'/metrics/' , each name, ' - ', each description ]).
	^ self okResponse: text! !

!NeoConsoleMetricDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 8/14/2015 22:05'!
responseForMetricNamed: metricName
	| metric |
	metric := NeoConsoleMetric named: metricName.
	^ self okResponse: metric value asString! !

Object subclass: #NeoConsoleTelnetServer
	instanceVariableNames: 'port process serverSocket'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Neo-Console-Core'!

!NeoConsoleTelnetServer commentStamp: 'SvenVanCaekenberghe 6/16/2015 19:23' prior: 0!
NeoConsoleTelnetServer offers a command line interface to a Pharo image.

Inspect:

  NeoConsoleTelnetServer new start.

In a terminal do:

  $ telnet localhost 4999

End with quit or an empty line.

Security warning: this service opens up your image for access to those with access to your local network (i.e. those logged in to your machine), without any further authenication, allowing them to do absolutely anything. Think and make sure that you know what you are doing.!

!NeoConsoleTelnetServer class methodsFor: 'public' stamp: 'SvenVanCaekenberghe 8/17/2015 13:56'!
startOn: port
	"self startOn: 4999"
	
	^ self new
		port: port;
		start! !

!NeoConsoleTelnetServer methodsFor: 'protocol' stamp: 'SvenVanCaekenberghe 6/16/2015 16:17'!
executeRequestResponseLoopOn: stream
	| in out command executeResult |
	in := ZnCharacterReadStream on: stream.
	out := ZnCharacterWriteStream on: stream.
	self writeLine: ('Neo Console ', SystemVersion current asString) to: out.
	[
		stream nextPutAll: '> '; flush.
		command := (self readInputFrom: in).
		executeResult := [ command execute ] 
			on: Error 
			do: [ :exception | command printString, ' failed with ', exception printString ].
		self writeOutput: command result to: out.
		executeResult notNil	] whileTrue! !

!NeoConsoleTelnetServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 2/11/2015 16:09'!
initializeServerSocket
	serverSocket := Socket newTCP.
	serverSocket setOption: 'TCP_NODELAY' value: 1.
	serverSocket 
		listenOn: self port 
		backlogSize: 10 
		interface: NetNameResolver loopBackAddress.
	serverSocket isValid
		ifFalse: [ self error: 'Cannot create socket on port ' , self port printString ]! !

!NeoConsoleTelnetServer methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 2/11/2015 15:49'!
isRunning
	"Return true when I am running"
	
	^ process notNil and: [ serverSocket notNil ]! !

!NeoConsoleTelnetServer methodsFor: 'protocol' stamp: 'SvenVanCaekenberghe 2/11/2015 16:00'!
listenLoop
	"We create a listening Socket, then wait for a connection.
	After each connection we also check that the listening Socket is still valid 
	- if not we just make a recursive call to this method to start over."

	self initializeServerSocket.
	[ [ 
		serverSocket isValid 
			ifFalse: [
				"will trigger #ifCurtailed: block and destroy socket"
				^ self listenLoop ].
		self serveConnectionsOn: serverSocket ] repeat ]

		ifCurtailed: [ self releaseServerSocket ]! !

!NeoConsoleTelnetServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/14/2015 16:15'!
port
	^ port ifNil: [ port := 4999 ]! !

!NeoConsoleTelnetServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/14/2015 16:15'!
port: integer
	port := integer! !

!NeoConsoleTelnetServer methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 8/17/2015 13:45'!
printOn: stream
	super printOn: stream.
	stream << $(.
	self isRunning ifTrue: [ stream << #running; space; << self port ].
	stream << $)! !

!NeoConsoleTelnetServer methodsFor: 'protocol' stamp: 'SvenVanCaekenberghe 2/24/2015 16:30'!
readInputFrom: stream
	| input lineReader line commandClass |
	lineReader := ZnFastLineReader on: stream.
	line := [ lineReader nextLine trimBoth ] on: ConnectionClosed, ConnectionTimedOut do: [ nil ].
	line isEmptyOrNil ifTrue: [ ^ NeoConsoleQuitCommand new ].
	commandClass := NeoConsoleCommand forLine: line.
	commandClass isSingleLine
		ifTrue: [ ^ commandClass new on: line; yourself ].
	input := String new writeStream.
	input nextPutAll: line; cr.
	[ 
		line := [ lineReader nextLine trimBoth ] on: ConnectionClosed, ConnectionTimedOut do: [ '' ].
		input nextPutAll: line; cr.
		line isEmpty ] whileFalse.
	^ commandClass new on: input contents; yourself! !

!NeoConsoleTelnetServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 2/11/2015 16:04'!
releaseServerSocket
	(Delay forMilliseconds: 10) wait.
	serverSocket destroy.
	serverSocket := nil! !

!NeoConsoleTelnetServer methodsFor: 'protocol' stamp: 'SvenVanCaekenberghe 2/11/2015 16:12'!
serveConnectionsOn: listeningSocket
	"We wait up to acceptWaitTimeout seconds for an incoming connection.
	If we get one we wrap it in a SocketStream and #executeRequestResponseLoopOn: on it"
		 
	| stream socket |
	socket := listeningSocket waitForAcceptFor: 300.
	socket ifNil: [ ^ self ].
	stream := self socketStreamOn: socket.
	[ [ [ self executeRequestResponseLoopOn: stream ]
		ensure: [ [ stream close ] on: Exception do: [ ] ] ]
			ifCurtailed: [ socket destroy ] ]
				forkAt: Processor lowIOPriority
				named: self workerProcessName ! !

!NeoConsoleTelnetServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/14/2015 16:21'!
serverProcessName
	^ self class name asString , ' port ' , self port asString! !

!NeoConsoleTelnetServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 2/11/2015 16:10'!
socketStreamOn: socket
	^ (ZdcSocketStream on: socket)
		binary;
		shouldSignal: true;
		autoFlush: false;
		timeout: self timeout;
		yourself! !

!NeoConsoleTelnetServer methodsFor: 'public' stamp: 'SvenVanCaekenberghe 2/11/2015 15:51'!
start
	self stop.
	process := [ [ self listenLoop ] repeat ] 
					forkAt: Processor highIOPriority 
					named: self serverProcessName! !

!NeoConsoleTelnetServer methodsFor: 'public' stamp: 'SvenVanCaekenberghe 2/11/2015 15:50'!
stop
	self isRunning ifFalse: [ ^ self ].
	process terminate. 
	process := nil.
! !

!NeoConsoleTelnetServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/16/2015 16:36'!
timeout
	^ 60! !

!NeoConsoleTelnetServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/11/2015 16:07'!
workerProcessName
	^ self serverProcessName, ' Connection Handler'! !

!NeoConsoleTelnetServer methodsFor: 'protocol' stamp: 'SvenVanCaekenberghe 8/14/2015 22:07'!
writeLine: string to: stream
	stream nextPutAll: string; lf; flush! !

!NeoConsoleTelnetServer methodsFor: 'protocol' stamp: 'SvenVanCaekenberghe 8/14/2015 22:07'!
writeOutput: string to: stream
	string lines
		do: [ :each | stream nextPutAll: each ]
		separatedBy: [ stream lf ].
	stream lf; flush! !
NeoConsoleMetric initialize!
